#ifndef __NLLIBC_SIGNAL_H_INCLUDED__
#define __NLLIBC_SIGNAL_H_INCLUDED__

#include <sys/types.h>

#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__DragonFly__) || defined(__APPLE__)

#define SIGHUP   1
#define SIGINT   2
#define SIGQUIT  3
#define SIGILL   4
#define SIGTRAP  5
#define SIGABRT  6
#define SIGKILL  9
#define SIGBUS  10
#define SIGSEGV 11
#define SIGPIPE 13
#define SIGTERM 15
#define SIGCHLD 20

typedef void (*sig_t)(int);

#define SIG_DFL ((sig_t) 0)
#define SIG_IGN ((sig_t) 1)
#define SIG_ERR ((sig_t)-1)

typedef struct {
  unsigned int __bits[4];
} sigset_t;

struct siginfo;

struct sigaction {
  union {
    void (*__sa_handler)(int);
    void (*__sa_sigaction)(int, struct siginfo *, void *);
  } __sigaction_u;
  int sa_flags;
  sigset_t sa_mask;
};
#define sa_handler   __sigaction_u.__sa_handler
#define sa_sigaction __sigaction_u.__sa_sigaction

#endif

#if defined(__linux__)

#define SIGHUP   1
#define SIGINT   2
#define SIGQUIT  3
#define SIGILL   4
#define SIGTRAP  5
#define SIGABRT  6
#define SIGKILL  9
#define SIGBUS   7
#define SIGSEGV 11
#define SIGPIPE 13
#define SIGTERM 15
#define SIGCHLD 17

typedef void (*sig_t)(int);

#define SIG_DFL ((sig_t) 0)
#define SIG_IGN ((sig_t) 1)
#define SIG_ERR ((sig_t)-1)

typedef struct {
  unsigned int sig[2];
} sigset_t;

struct siginfo;

#if defined(__i386__) || defined(__arm__) || defined(__aarch64__) || defined(__sh__)
struct sigaction {
  union {
    void (*__sa_handler)(int);
    void (*__sa_sigaction)(int, struct siginfo *, void *);
  } __sigaction_u;
  sigset_t sa_mask;
  unsigned long sa_flags;
  void (*sa_restorer)(void);
};
#endif
#if defined(__amd64__)
struct sigaction {
  union {
    void (*__sa_handler)(int);
    void (*__sa_sigaction)(int, struct siginfo *, void *);
  } __sigaction_u;
  unsigned long sa_flags;
  sigset_t sa_mask;
};
#endif
#if defined(__mips__)
struct sigaction {
  unsigned int sa_flags;
  union {
    void (*__sa_handler)(int);
    void (*__sa_sigaction)(int, struct siginfo *, void *);
  } __sigaction_u;
  sigset_t sa_mask;
};
#endif
#if defined(__PPC__)
struct sigaction {
  union {
    void (*__sa_handler)(int);
    void (*__sa_sigaction)(int, struct siginfo *, void *);
  } __sigaction_u;
  unsigned long sa_flags;
  void (*sa_restorer)(void);
  sigset_t sa_mask;
};
#endif
#define sa_handler   __sigaction_u.__sa_handler
#define sa_sigaction __sigaction_u.__sa_sigaction

#endif

int kill(pid_t pid, int sig);

int sigaction(int sig, const struct sigaction *act, struct sigaction *oact);
int sigprocmask(int how, const sigset_t *set, sigset_t *oset);

int killpg(pid_t pgrp, int sig);

#endif
